@@ -10,8 +10,6 @@
 -- You might consider trying 'Text.Coffee', which compiles down to Javascript.
 --
 -- Further reading: <http://www.yesodweb.com/book/templates>
---
--- This module is currently in an identity crisis. Originally called Julius, now being changed to just Javascript (shakespeare-javascript)
 module Text.Julius
     ( -- * Functions
       -- ** Template-Reading Functions
@@ -29,9 +27,11 @@ module Text.Julius
       -- * Datatypes
     , JavascriptUrl
     , Javascript (..)
+    , RawJavascript (..)
 
       -- * Typeclass for interpolated variables
     , ToJavascript (..)
+    , RawJS (..)
 
       -- ** Rendering Functions
     , renderJavascript
@@ -39,6 +39,8 @@ module Text.Julius
 
       -- ** internal, used by 'Text.Coffee'
     , javascriptSettings
+      -- ** internal
+    , juliusUsedIdentifiers
     ) where
 
 import Language.Haskell.TH.Quote (QuasiQuoter (..))
@@ -48,6 +50,8 @@ import Data.Monoid
 import qualified Data.Text as TS
 import qualified Data.Text.Lazy as TL
 import Text.Shakespeare
+import Data.Aeson (Value)
+import Data.Aeson.Encode (fromValue)
 
 renderJavascript :: Javascript -> TL.Text
 renderJavascript (Javascript b) = toLazyText b
@@ -69,21 +73,43 @@ newtype Javascript = Javascript { unJavascript :: Builder }
 -- | Return type of template-reading functions.
 type JavascriptUrl url = (url -> [(TS.Text, TS.Text)] -> TS.Text) -> Javascript
 
+asJavascriptUrl :: JavascriptUrl url -> JavascriptUrl url
+asJavascriptUrl = id
+
 -- | A typeclass for types that can be interpolated in CoffeeScript templates.
 class ToJavascript a where
     toJavascript :: a -> Builder
+#if 0
 instance ToJavascript [Char] where toJavascript = fromLazyText . TL.pack
 instance ToJavascript TS.Text where toJavascript = fromText
 instance ToJavascript TL.Text where toJavascript = fromLazyText
+instance ToJavascript Javascript where toJavascript = unJavascript
+instance ToJavascript Builder where toJavascript = id
+#endif
+instance ToJavascript Value where toJavascript = fromValue
+
+newtype RawJavascript = RawJavascript Builder
+instance ToJavascript RawJavascript where
+    toJavascript (RawJavascript a) = a
+
+class RawJS a where
+    rawJS :: a -> RawJavascript
+
+instance RawJS [Char] where rawJS = RawJavascript . fromLazyText . TL.pack
+instance RawJS TS.Text where rawJS = RawJavascript . fromText
+instance RawJS TL.Text where rawJS = RawJavascript . fromLazyText
+instance RawJS Builder where rawJS = RawJavascript
 
 javascriptSettings :: Q ShakespeareSettings
 javascriptSettings = do
   toJExp <- [|toJavascript|]
   wrapExp <- [|Javascript|]
   unWrapExp <- [|unJavascript|]
+  asJavascriptUrl' <- [|asJavascriptUrl|]
   return $ defaultShakespeareSettings { toBuilder = toJExp
   , wrap = wrapExp
   , unwrap = unWrapExp
+  , modifyFinalValue = Just asJavascriptUrl'
   }
 
 js, julius :: QuasiQuoter
@@ -114,3 +140,8 @@ juliusFileDebug = jsFileReload
 {-# DEPRECATED juliusFileDebug "Please use juliusFileReload instead." #-}
 jsFileDebug = jsFileReload
 {-# DEPRECATED jsFileDebug "Please use jsFileReload instead." #-}
+
+-- | Determine which identifiers are used by the given template, useful for
+-- creating systems like yesod devel.
+juliusUsedIdentifiers :: String -> [(Deref, VarType)]
+juliusUsedIdentifiers = shakespeareUsedIdentifiers defaultShakespeareSettings
