@@ -1,5 +1,5 @@
-import Test.Hspec.Monadic
+import Test.Hspec
 import ShakespeareJsTest (specs)
 
 main :: IO ()
-main = hspecX specs
+main = hspec specs
