@@ -4,8 +4,7 @@
 module ShakespeareJsTest (specs) where
 
 import Test.HUnit hiding (Test)
-import Test.Hspec.Monadic
-import Test.Hspec.HUnit ()
+import Test.Hspec
 
 import Prelude hiding (reverse)
 #ifdef TEST_COFFEE
@@ -19,6 +18,7 @@ import qualified Data.List
 import qualified Data.List as L
 import Data.Text (Text, pack, unpack)
 import Data.Monoid (mappend)
+import Data.Aeson (toJSON)
 
 join :: [String] -> String
 #ifdef TEST_COFFEE
@@ -27,12 +27,12 @@ join l = (intercalate ";\n" l)
 join = intercalate "\n"
 #endif
 
-specs :: Specs
+specs :: Spec
 specs = describe "shakespeare-js" $ do
   it "julius" $ do
     let var = "x=2"
     let urlp = (Home, [(pack "p", pack "q")])
-    flip jelper [quote|['שלום', @{Home}, #{var}, '@?{urlp}', ^{jmixin} ]|]
+    flip jelper [quote|['שלום', @{Home}, #{rawJS var}, '@?{urlp}', ^{jmixin} ]|]
       $ intercalate " "
         [ "['שלום',"
         , "url, " ++ var ++ ","
@@ -84,7 +84,7 @@ specs = describe "shakespeare-js" $ do
 #ifdef TEST_COFFEE
          [quote|[%{Data.List.reverse foo}, %{L.reverse foo}, %{show double}, %{show int}]|]
 #else
-         [quote|[#{Data.List.reverse foo}, #{L.reverse foo}, #{show double}, #{show int}]|]
+         [quote|[#{rawJS $ Data.List.reverse foo}, #{rawJS $ L.reverse foo}, #{rawJS $ show double}, #{rawJS $ show int}]|]
 #endif
 
 
@@ -97,8 +97,12 @@ specs = describe "shakespeare-js" $ do
 
   it "dollar operator" $ do
     let val = (1 :: Int, (2 :: Int, 3 :: Int))
-    jelper "2" [quote|#{ show $ fst $ snd val }|]
-    jelper "2" [quote|#{ show $ fst $ snd $ val}|]
+    jelper "2" [quote|#{ rawJS $ show $ fst $ snd val }|]
+    jelper "2" [quote|#{ rawJS $ show $ fst $ snd $ val}|]
+
+  it "empty file" $ jelper "" [quote||]
+
+  it "JSON data" $ jelper "\"Hello \\\"World!\\\"\"" [julius|#{toJSON "Hello \"World!\""}|]
 
 
 
